package main

import (
	"fmt"
	"log"
	"os"

	neuralnetwork "./neuralnetworkbase"
	teach "./teach"
)

func main() {
	sizes := []int{13, 14, 14, 3}
	nn, _ := neuralnetwork.NewNeuralNetwork(sizes, 200, neuralnetwork.NewPlusRPropInitializer(neuralnetwork.RPropConfig{
		NuPlus:   1.2,
		NuMinus:  0.8,
		DeltaMax: 50.0,
		DeltaMin: 0.000001,
	}))

	// nn, _ := neuralnetwork.NewNeuralNetwork(sizes, 200, neuralnetwork.NewBackPropInitializer(0.1))

	// for i := 0; i < nn.Count; i++ {
	// 	if i > 0 {
	// 		fmt.Printf("Weights before:\n%v\n\n", mat.Formatted(nn.Weights[i], mat.Prefix(""), mat.Excerpt(0)))
	// 		fmt.Printf("Biases before:\n%v\n\n", mat.Formatted(nn.Biases[i], mat.Prefix(""), mat.Excerpt(0)))
	// 		fmt.Printf("Z before:\n%v\n\n", mat.Formatted(nn.Z[i], mat.Prefix(""), mat.Excerpt(0)))
	// 	}
	// 	fmt.Printf("A before:\n%v\n\n", mat.Formatted(nn.A[i], mat.Prefix(""), mat.Excerpt(0)))
	// }

	teacher := teach.NewTextDataReader("./wine.data", 5)
	nn.Teach(teacher)

	// for i := 0; i < nn.Count; i++ {
	// 	if i > 0 {
	// 		fmt.Printf("Weights after:\n%v\n\n", mat.Formatted(nn.Weights[i], mat.Prefix(""), mat.Excerpt(0)))
	// 		fmt.Printf("Biases after:\n%v\n\n", mat.Formatted(nn.Biases[i], mat.Prefix(""), mat.Excerpt(0)))
	// 		fmt.Printf("Z after:\n%v\n\n", mat.Formatted(nn.Z[i], mat.Prefix(""), mat.Excerpt(0)))
	// 	}
	// 	fmt.Printf("A after:\n%v\n\n", mat.Formatted(nn.A[i], mat.Prefix(""), mat.Excerpt(0)))
	// }

	outFile, err := os.OpenFile("./data", os.O_CREATE|os.O_TRUNC|os.O_WRONLY, 0666)
	if err != nil {
		log.Fatal(err)
	}
	defer outFile.Close()
	nn.SaveState(outFile)
	outFile.Close()

	failCount := 0
	teacher.Reset()
	for teacher.NextValidator() {
		dataSet, expect := teacher.GetValidator()
		index, _ := nn.Predict(dataSet)
		if expect.At(index, 0) != 1.0 {
			failCount++
			fmt.Printf("Fail: %v, %v\n\n", teacher.ValidationIndex(), expect.At(index, 0))
		}
	}
	fmt.Printf("Fail count: %v\n\n", failCount)

	nn = &neuralnetwork.NeuralNetwork{}
	inFile, err := os.Open("./data")
	if err != nil {
		log.Fatal(err)
	}
	defer inFile.Close()
	nn.LoadState(inFile)
	inFile.Close()

	failCount = 0
	teacher.Reset()
	for teacher.NextValidator() {
		dataSet, expect := teacher.GetValidator()
		index, _ := nn.Predict(dataSet)
		if expect.At(index, 0) != 1.0 {
			failCount++
			fmt.Printf("Fail: %v, %v\n\n", teacher.ValidationIndex(), expect.At(index, 0))
		}
	}

	fmt.Printf("Fail count: %v\n\n", failCount)
}
